function [th,intv,P]=linfit(D,ilow,ihi)
% function [th,intv,P]=linfit(D,ilow,ihi)
%
% Function for fitting a linear segment of unknown length
% to data.
%
% y = a x_1 + b x_2 + c x_3 ... + e_t
%
% Disturbances e_t are assumed to be Gaussian distributed
% with unknown variance.
%
% D    Data matrix composed of columns: D = [y, x_1, x_2, x_3...]
% ilow Index of the first record to be processed [default=1]
% ihi  Index of the last record to be processed [defualt=end]
%
% output:
%   th   estimate of linera coefficients [a,b,c...]
%   intv estimate of the interval [tl,th]
%   P    probability matrix of P(i=tl,j=th) 

if nargin <3
    ihi = size(D,1);
    if nargin <2
        ilow = 1;
    end
end

% SCALE so that (max(y)-min(y)=1)
if 0
    SC = max(D(:,1))-min(D(:,1));
    D = D/SC;
    
end
dim = size(D,2);
ndat = ihi-ilow+1;
llUni = -log(max(D(:,1))-min(D(:,1)))+log(3);


% Initial phase
% ==============

% prior
%     V0 = 1e-2*eye(dim);
%     V0(1,1) = 1e-4;
%     nu0 = 0.1;
    %
    V0 = zeros(dim);
    nu0=0;
%[Eth,Cth,cove]=ld2ls(ldform(V0),nu0)    
% matrix of likelihoods
LL = ndat*llUni*ones(ihi); % everything is uniformly distributed
rgr = zeros(dim,1);

for i=ilow:ihi % first cycle
    V = V0;
    nu = nu0;
    for j = i:ihi
        rgr = D(j,:);
        V = V + rgr'*rgr;
        nu = nu +1;

        if j>=i+dim-1
            % save log-likelihood of data
            Vaa = V(2:end,2:end);
            iVaa = inv(Vaa);
            Lam = V(1,1) - V(1,2:end)*iVaa*V(2:end,1);
            % for numerical stability
            if Lam<1e-13
                Lam = 1e-13;
            end;
            LL(i,j) = gammaln(0.5*nu) -0.5*nu*log(Lam) ...
                - 0.5*1*log(det(Vaa)) + 1/2*nu*log(2) ...
                +1/2*(2-nu)*log(pi)+...
                llUni*(ndat-nu+nu0);
            if ~isreal(LL(i,j))
                keyboard
            end
            % other terms omitted since they are equal
            % for all considered hypotheses            
        end
    end
end

% add probability of the remaining points beeing uniformly distributed 
% between min(D) and max(D).

[m1,i1]=max(LL);
[m2,i2]=max(m1);
i1 = i1(i2);

% figure; %surf(LL); 
% surf(exp(LL-max(max(LL))))

V = V0;
nu = nu0;

for i=i1:i2 % firstndat cycle
    rgr = D(i,:);
    V = V + rgr'*rgr;
    nu = nu +1;
end

intv = [i1 i2];
th = inv(V(2:end,2:end))*V(2:end,1);

P = exp(LL-max(max(LL)));
P = P/sum(sum(P)); % normalize;
